"""Test runner package - orchestrates action-based testing with MCP tools.

This package provides utilities for:
- Variable injection and parameter substitution
- Tool parameter sanitization (including Asana-specific logic)
- Action sequence execution
- Test orchestration and evaluation

Public API:
- run_tests: Run a batch of dataset examples
- execute_action_plan: Execute a single action plan (for provisioning, setup, etc.)
"""


